/* ***************************************************************************
   *                                                                         *
   *   Kindermann                        Wahlpflichtkurs Produktentwicklung  *
   *                                     Schuljahr: 2015-16                  *
   *                                                                         *
   *                                                                         *
   *                            Attiny85 ADC-Reader                          *
   *                   (based on Little Wire Firmware 1.3)                   *
   *                                                                         *
   ***************************************************************************/


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <util/atomic.h>
#include <stdlib.h>
#include "usbdrv.h"
#include "digital.h"
#define delayMicroseconds(value) _delay_us(value);
#define sbi(register,bit) (register|=(1<<bit))
#define cbi(register,bit) (register&=~(1<<bit))

enum
{
	// Generic requests
	USBTINY_ECHO,		// echo test	0
	USBTINY_READ,		// read byte (wIndex:address)	1
	USBTINY_WRITE,		// write byte (wIndex:address, wValue:value)	2
	USBTINY_CLR,		// clear bit (wIndex:address, wValue:bitno)	3
	USBTINY_SET,		// set bit (wIndex:address, wValue:bitno)	4
	// Programming requests
	USBTINY_POWERUP,	// apply power (wValue:SCK-period, wIndex:RESET)	5
	USBTINY_POWERDOWN,	// remove power from chip	6
	USBTINY_SPI,		// issue SPI command (wValue:c1c0, wIndex:c3c2)	7
	USBTINY_POLL_BYTES,	// set poll bytes for write (wValue:p1p2)	8
	USBTINY_FLASH_READ,	// read flash (wIndex:address)	9
	USBTINY_FLASH_WRITE,	// write flash (wIndex:address, wValue:timeout)	10
	USBTINY_EEPROM_READ,	// read eeprom (wIndex:address)	11
	USBTINY_EEPROM_WRITE,	// write eeprom (wIndex:address, wValue:timeout)	12
	// Additional requests - ihsanKehribar
	USBTINY_PIN_SET_INPUT,	//	13
	USBTINY_PIN_SET_OUTPUT, //	14
	USBTINY_READ_ADC,	//	15
	USBTINY_SETUP_PWM, //16
	USBTINY_UPDATE_PWM_COMPARE, // 17
	USBTINY_PIN_SET_HIGH, // 18
	USBTINY_PIN_SET_LOW, // 19
	USBTINY_PIN_READ, // 20
	USBTINY_SINGLE_SPI, // 21
	USBTINY_CHANGE_PWM_PRESCALE, // 22
	USBTINY_SETUP_SPI, // 23
	USBTINY_SETUP_I2C, // 24
	USBTINY_I2C_BEGIN_TX, // 25
	USBTINY_I2C_ADD_BUFFER, // 26
	USBTINY_I2C_SEND_BUFFER, // 27
	USBTINY_SPI_ADD_BUFFER, // 28
	USBTINY_SPI_SEND_BUFFER, // 29
	USBTINY_I2C_REQUEST_FROM, // 30
	USBTINY_SPI_UPDATE_DELAY, // 31
	USBTINY_STOP_PWM, // 32
 	USBTINY_DEBUG_SPI, // 33
	USBTINY_VERSION_QUERY, // 34	
};

#define	PORT PORTB
#define	DDR	DDRB
#define	PIN	PINB

uint8_t* EE_addr = (uint8_t*)32;

#define	POWER_MASK	0x00
#define	GND_MASK	0x00

#define	RESET_MASK	(1 << 5)
#define	SCK_MASK	(1 << 2)
#define	MISO_MASK	(1 << 1)
#define	MOSI_MASK	(1 << 0)

#define SPI_BUFFER_SIZE 16
#define checkBit(reg,bit) (reg&(1<<bit)) 

// ----------------------------------------------------------------------
// Local data
// ----------------------------------------------------------------------
static	unsigned	address;	// read/write address
static	unsigned	timeout;	// write timeout in usec
static	uint16_t	SPI_DELAY=10;// in microseconds. USI driven SPI mode
static	uint16_t	I2C_DELAY=0;// in microseconds. USI driven SPI mode
// ----------------------------------------------------------------------
volatile	uint8_t		sendBuffer[9];
volatile	uint8_t		rxBuffer[8];
static uint8_t t,q;
static uint8_t counter=0;
static uint8_t cmp0,cmp1,cmp2,compare0,compare1,compare2;
static uint8_t adcSetting=0;
volatile uint8_t 	jobState=0;
#define DATA_PIN 2


/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */
uchar	usbFunctionSetup(uchar data[8])
{
// ----------------------------------------------------------------------
// Handle a non-standard SETUP packet.
// ----------------------------------------------------------------------
	uchar	bit;
	uchar	mask;
	uchar*	addr;
	uchar	req;

	// Generic requests
	req = data[1];
	addr = (uchar*) (int) data[4];
	bit = data[2] & 7;
	mask = 1 << bit;

	// Programming requests
	address = * (unsigned*) & data[4];
	timeout = * (unsigned*) & data[2];

	if( req == USBTINY_READ_ADC ) // 15
	{
		// unsigned char temp=DDR;

		if(data[2]==0) // Read ADC from RESET pin
		{
			ADMUX = adcSetting | 0; // Set the desired channel
			sbi(DIDR0,ADC0D); // Disable digital buffer ...
			// DDR &= ~(1<<5);
		}
		else if(data[2]==1) // Read ADC from SCK pin
		{
			ADMUX = adcSetting | 1; // Set the desired channel	
			sbi(DIDR0,ADC1D); // Disable digital buffer ...
			// DDR &= ~(1<<2);
		}
		else if(data[2]==2) // Read ADC from internal Temperature sensor
		{
			ADMUX=0b10001111;
		}		
		sbi(ADCSRA,ADSC); // Start conversion
		while(checkBit(ADCSRA,ADSC)); // Wait for conversion	
		data[0]=(unsigned char)ADCL;
		data[1]=(unsigned char)ADCH;
		usbMsgPtr = data;
		// DDR=temp;
		DIDR0=0x00;
		return 2;
	}
	
	if ( req == 35 ) /* init ADC */
	{
		sbi(ADCSRA,ADEN); // Enable the ADC peripheral
		ADCSRA |= data[2]; // Least significant three bits are prescale settings
		switch(data[3])
		{
			case 0: // Use VCC as Vref.
				adcSetting = 0;
				break;
			case 1: // 1.1V 
				sbi(adcSetting,REFS1);
				break;
			case 2: // 2.56V
				sbi(adcSetting,REFS1);
				sbi(adcSetting,REFS2);
				break;
		}
		return 0;
	}
	if (req == 40) /* read the results */
	{	
		usbMsgPtr = sendBuffer;
		return sendBuffer[8];
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue; 
}

void usbEventResetReady(void)
{
    calibrateOscillator();
    eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}

/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */
int main(void) {
	uchar   i;
	uchar   calibrationValue;
	
	DDRB  = RESET_MASK;
	PORTB = RESET_MASK;

    calibrationValue = eeprom_read_byte(0); /* calibration value from last time */
    if(calibrationValue != 0xff){
        OSCCAL = calibrationValue;
    }

    //initSerialNumber();
    
    usbDeviceDisconnect();
    for(i=0;i<20;i++){  /* 300 ms disconnect */
        _delay_ms(15);
    }
    usbDeviceConnect();

	I2C_DELAY = 5;
	SPI_DELAY = 0;
	
    wdt_enable(WDTO_1S);

	cmp0=0;	cmp1=0;	cmp2=0;	
	
	compare0=0;	compare1=0;	compare2=0;	
	
	counter=0;
	
	jobState=0;

    usbInit();
    sei();

	pinMode(B,0,INPUT);
	internalPullup(B,0,DISABLE);
	pinMode(B,1,INPUT);
	internalPullup(B,1,DISABLE);
	pinMode(B,2,INPUT);
	internalPullup(B,2,DISABLE);		
	pinMode(B,5,INPUT);
	internalPullup(B,5,DISABLE);
	
    for(;;)
	{
        wdt_reset();
        usbPoll();				

		//-----------------------------------------------------------------------------
		// parse rxBuffer and store the result in the sendBuffer		
		// sendBuffer[8] represents the length of the buffer
		//-----------------------------------------------------------------------------
		switch(jobState)
		{
			case 0: /* idle ... */
			break;
			case 1: /* debug spi */
				DDRB |= MOSI_MASK;
				DDRB &= ~MISO_MASK;
				DDRB |= SCK_MASK;
				PORTB |= SCK_MASK;
				sendBuffer[0]=0;
				t = rxBuffer[0];
				for	( q = 0x80; q; q >>= 1 )
				{			
					PORT &= ~MOSI_MASK; /* Clear send data */													
					for(i=0;i<SPI_DELAY;i++) _delay_us(1); /* Small delay */			
					if	( t & q ) { PORTB |= MOSI_MASK; } /* Send the data */			
					PORT &= ~SCK_MASK; /* High to low edge */
					sendBuffer[0]<<=1; /* Shift the register */						
					sendBuffer[0]+=((PINB&(MISO_MASK))>>1); /* Sample the data */					
					for(i=0;i<SPI_DELAY;i++) _delay_us(1); /* Small delay */
					PORT |= SCK_MASK; /* Low to high edge */										
				}				
				sendBuffer[8]=1; // length				
				// --------------------------------------------------------------------
				jobState=0;
			break;
			case 2: /* onewire reset pulse */											
				pinMode(B,DATA_PIN,OUTPUT);
				digitalWrite(B,DATA_PIN,LOW);				// Drive the bus low
				delayMicroseconds(480); 					// delay 480 microsecond (us)		
				ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
				{	
					digitalWrite(B,DATA_PIN,HIGH);			// Release the bus				
					delayMicroseconds(70);					// delay 70 microsecond (us)
					pinMode(B,DATA_PIN,INPUT);				
					sendBuffer[0]=!(digitalRead(B,DATA_PIN)>>2);// Sample for presence pulse from slave							
				}
				delayMicroseconds(410);						// delay 410 microsecond (us)										
				pinMode(B,DATA_PIN,OUTPUT);	   				// Release the bus
				digitalWrite(B,DATA_PIN,HIGH);				
				sendBuffer[8]=1;				
				jobState=0;
			break;
			case 3: /* onewire send byte */				
				q=rxBuffer[0];
				pinMode(B,DATA_PIN,OUTPUT);
				for (i=0;i<8;i++)
				{		
					ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
					{		
						digitalWrite(B,DATA_PIN,LOW);		// Drive the bus low
			
						if ( q & 0x01)
						{							
							delayMicroseconds(6);			// delay 6 microsecond (us)						
							digitalWrite(B,DATA_PIN,HIGH);	// Release the bus
							delayMicroseconds(64);			// delay 64 microsecond (us)					
						}
						else
						{						
							delayMicroseconds(60);			// delay 60 microsecond (us)					
							digitalWrite(B,DATA_PIN,HIGH);	// Release the bus
							delayMicroseconds(10);			// delay 10 microsecond (us)										
						}
						q >>= 1;							// shift the data byte for the next bit to send
					}
				}					
				jobState=0;
			break;
			case 4: /* onewire read byte */
				sendBuffer[0]=0;
				for(i=0;i<8;i++)
				{
					sendBuffer[0] >>= 1; 					// shift the result to get it ready for the next bit to receive
					pinMode(B,DATA_PIN,OUTPUT);				
					ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
					{	
						digitalWrite(B,DATA_PIN,LOW);		// Drive the bus low
						delayMicroseconds(6);				// delay 6 microsecond (us)
						digitalWrite(B,DATA_PIN,HIGH);		// Release the bus
						delayMicroseconds(10);				// delay 9 microsecond (us)					
						pinMode(B,DATA_PIN,INPUT);				
						q = (digitalRead(B,DATA_PIN)>>2);	// Read the status of OW_PIN
					}
					delayMicroseconds(55);					// delay 55 microsecond (us)					
					if (q)	sendBuffer[0] |= 0x80;			// if result is one, then set MS-bit
				}				
				sendBuffer[8]=1;
				jobState=0;
			break; 
			case 5: /* onewire read bit */
				pinMode(B,DATA_PIN,OUTPUT);	
				ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
				{
					digitalWrite(B,DATA_PIN,LOW);			// Drive the bus low
					delayMicroseconds(6);					// delay 6 microsecond (us)
					digitalWrite(B,DATA_PIN,HIGH);			// Release the bus
					delayMicroseconds(10);					// delay 9 microsecond (us)				
					pinMode(B,DATA_PIN,INPUT);				
					sendBuffer[0]=(digitalRead(B,DATA_PIN)>>2); // Read the status of OW_PIN
				}
				sendBuffer[8]=1;
				jobState=0;
			break;
			case 6: /* onewire write bit */
				pinMode(B,DATA_PIN,OUTPUT);
				ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
				{		
					digitalWrite(B,DATA_PIN,LOW);			// Drive the bus low		
					if ( rxBuffer[0] & 0x01)
					{							
						delayMicroseconds(6);				// delay 6 microsecond (us)						
						digitalWrite(B,DATA_PIN,HIGH);		// Release the bus
						delayMicroseconds(64);				// delay 64 microsecond (us)					
					}
					else
					{						
						delayMicroseconds(60);				// delay 60 microsecond (us)					
						digitalWrite(B,DATA_PIN,HIGH);		// Release the bus
						delayMicroseconds(10);				// delay 10 microsecond (us)										
					}
				}
				jobState=0;
			break;
			case 7:  /* multiple spi send and receive */
				DDRB |= MISO_MASK;
				DDRB &= ~MOSI_MASK;
				DDRB |= SCK_MASK;
				PORTB &= ~SCK_MASK;
				t=(1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
				USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK);
				if(rxBuffer[0])	PORT &= ~(1<<5); // auto chip select?
				for(q=0;q<rxBuffer[1];q++)
				{
					USIDR=rxBuffer[2+q];
					USISR=(1<<USIOIF);
					do {
						USICR = t;						
						for(i=0;i<SPI_DELAY;i++)	_delay_us(1);
					} while ((USISR & (1<<USIOIF))==0);	
					sendBuffer[q]=USIDR;
				}
				if(rxBuffer[0])	PORT |= (1<<5); // auto chip select?
				sendBuffer[8]=q;
				jobState=0;
			break;			
		}	
    }
    return 0;
}

